;;; File: "debugger.scm"

(include "rdi.scm")

;;;----------------------------------------------------------------------------

(define console-window-num 0)

(define (new-console-window-num)
  (let ((x (+ console-window-num 1)))
    (set! console-window-num x)
    x))

(define (open-console-window title)
  (let ((tcp-port (+ 9000 (new-console-window-num))))
    (let ((window
           (open-process
            (list path: "xterm"
                  arguments: (list "-title"
                                   title
                                   "-e"
                                   "gsi"
                                   ;; "-:tE"
                                   "pump.scm"
                                   (number->string tcp-port))))))
      (let loop ()
        (let ((port
               (with-exception-catcher
                (lambda (e)
                  #f)
                (lambda ()
                  (let ((port (open-tcp-client
                               (list server-address: "localhost"
                                     port-number: tcp-port))))
                    (tcp-client-peer-socket-info port)
                    port)))))
          (if (not port)
              (begin
                (thread-sleep! .1) ;; wait until the pump starts
                (loop))
              port))))))

;;;-----------------------------------------------------------------------------

(define rdi
  (rdi-create-server #f))

(rdi-force-connection rdi);;;;;;;;;;;;;

(define (rdi-function fn)
  (case fn
    ((register-console)
     rdi-register-console)
    ((console-output)
     rdi-console-output)
    ((close-console)
     rdi-close-console)
    (else
     (error "unknown function"))))

;;;-----------------------------------------------------------------------------

(define rdi-console-table (make-table))

(define (rdi-register-console console-id title)
  (let ((console-port (open-console-window title)))
    (table-set! rdi-console-table console-id console-port)
    (rdi-console-input-pump-start! console-id console-port)
    #f))

(define (rdi-console-output console-id output)
  (let ((console-port
         (table-ref rdi-console-table console-id #f)))
    (if console-port
        (begin
          (display output console-port)
          (force-output console-port))))
  #t)

(define (rdi-console-input-pump-start! console-id console-port)
  (thread-start!
   (make-thread
    (lambda ()
      (let* ((buflen 1000)
             (buf (make-string buflen)))
        (let loop ()
          (let ((n (read-substring buf 0 buflen console-port 1)))
            (if (> n 0)
                (begin
                  (rdi-remote-call rdi
                                   'console-input
                                   console-id
                                   (substring buf 0 n))
                  (loop))))))))))

(define (rdi-close-console console-id)
  (let ((console-port
         (table-ref rdi-console-table console-id #f)))
    (if console-port
        (close-output-port console-port)))
  #t)

;;;-----------------------------------------------------------------------------

(thread-join! (rdi-writer-thread rdi))
